/*
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * Copyright 2012-2023 the original author or authors.
 */
package org.assertj.core.util;

/**
 * @author Mariusz Smykula
 */
public class Hexadecimals {

  protected static final char[] HEX_ARRAY = "0123456789ABCDEF".toCharArray();

  public static String byteToHexString(byte b) {
    int v = b & 0xFF;
    return new String(new char[] { HEX_ARRAY[v >>> 4], HEX_ARRAY[v & 0x0F] });
  }

  public static String toHexString(byte... bytes) {
    StringBuilder stringBuilder = new StringBuilder();
    for (byte b : bytes)
      stringBuilder.append(byteToHexString(b));

    return stringBuilder.toString();
  }

  private Hexadecimals() {}
}
