/*
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * Copyright 2012-2023 the original author or authors.
 */
package org.assertj.core.error;

import org.assertj.core.presentation.PredicateDescription;

public class AnyElementShouldMatch extends BasicErrorMessageFactory {
  private static final String NON_MATCHING_ELEMENT = "%nExpecting any elements of:%n  %s%nto match %s predicate but none did.";

  public static ErrorMessageFactory anyElementShouldMatch(Object actual, PredicateDescription predicateDescription) {
    return new AnyElementShouldMatch(actual, predicateDescription);
  }

  private AnyElementShouldMatch(Object actual, PredicateDescription predicateDescription) {
    super(NON_MATCHING_ELEMENT, actual, predicateDescription);
  }
}
