/*
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * Copyright 2012-2023 the original author or authors.
 */
package org.assertj.core.api.iterator;

import java.util.Arrays;

import org.assertj.core.api.IterableAssert;
import org.assertj.core.api.IteratorAssert;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

/**
 * @author Sára Juhošová
 */
class IteratorAssert_toIterable_Test {

  private IteratorAssert<Integer> assertions;

  @BeforeEach
  void setup() {
    assertions = new IteratorAssert<>(Arrays.stream(new Integer[] { 5, 13, 42, 100 }).iterator());
  }

  @Test
  void should_return_equivalent_iterable() {
    IterableAssert<Integer> returned = assertions.toIterable();
    returned.containsExactly(5, 13, 42, 100);
  }

}
