/*
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * Copyright 2012-2023 the original author or authors.
 */
package org.example.test;

import java.io.IOException;
import java.util.Objects;

import org.assertj.core.api.AbstractAssert;

public class MyProjectClassAssert extends AbstractAssert<MyProjectClassAssert, MyProjectClass> {

  public MyProjectClassAssert(MyProjectClass actual) {
    super(actual, MyProjectClassAssert.class);
  }

  public MyProjectClassAssert hasValue(Object value) throws Exception {
    if (value == null) throw new IOException("does not mean anything, it's just for the test");
    if (!Objects.equals(actual.getValue(), value)) {
      failWithMessage("Expecting value to be <%s> but was <%s>:", value, actual.getValue());
    }
    return this;
  }

}