/*
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * Copyright 2012-2023 the original author or authors.
 */
package org.assertj.core.api.test;

import static java.lang.String.format;

public class ComparableExample implements Comparable<ComparableExample> {

  private int id;

  public ComparableExample(int id) {
    this.id = id;
  }

  @Override
  public int compareTo(ComparableExample that) {
    return this.id - that.id;
  }

  @Override
  public String toString() {
    return format("ComparableExample [id=%s]", id);
  }

}