/*
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * Copyright 2012-2023 the original author or authors.
 */
package org.assertj.core.error;

import java.math.BigDecimal;

public class ShouldHaveScale extends BasicErrorMessageFactory {

  public static ErrorMessageFactory shouldHaveScale(BigDecimal actual, int expectedScale) {
    return new ShouldHaveScale(actual, expectedScale);
  }

  private ShouldHaveScale(BigDecimal actual, int expectedScale) {
    super("%nexpecting actual %s to have a scale of:%n" +
          "  %s%n" +
          "but had a scale of:%n" +
          "  %s",
          actual, expectedScale, actual.scale());
  }
}
