/*
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * Copyright 2012-2023 the original author or authors.
 */
package org.assertj.core.api;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.BDDAssertions.then;

import java.util.Spliterator;
import java.util.stream.Stream;

import org.assertj.core.test.StringSpliterator;
import org.junit.jupiter.api.Test;

/**
 * Tests for <code>{@link Assertions#assertThat(Spliterator)}</code>.
 *
 * @author William Bakker
 */
class Assertions_assertThat_with_Spliterator_Test {

  private final Spliterator<?> stringSpliterator = new StringSpliterator();

  @Test
  void should_create_Assert() {
    // WHEN
    SpliteratorAssert<String> assertion = assertThat(Stream.of("Luke", "Leia").spliterator());
    // THEN
    then(assertion).isNotNull();
  }

  @Test
  void should_pass_actual() {
    // WHEN
    SpliteratorAssert<?> assertion = assertThat(stringSpliterator);
    // THEN
    then(assertion.actual).isSameAs(stringSpliterator);
  }

}
