/*
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 * Copyright 2012-2023 the original author or authors.
 */
package org.assertj.core.test;

/**
 * Object for test with a private inaccessible field set a unique value.
 */
public class TestClassWithRandomId {

  private final long id;
  public final String s;
  public final int n;

  public TestClassWithRandomId(String s, final int n) {
	this.s = s;
	this.n = n;
	this.id = System.currentTimeMillis();
	// sleep to avoid creating two objects with same id
	try {
	  Thread.sleep(5);
	} catch (InterruptedException e) {
	  // nothing to do
	}
  }

  @Override
  public String toString() {
	return "id=" + id;
  }

  @SuppressWarnings("unused")
  private long getId() {
	return id;
  }

}
